package bioinformatica.programmazione.esami.giugno.terzaparte;

/**
 * Classe concreta che rappresenta insiemi non modificabili di stringe.
 * 
 * @author mace
 * @version 1.0
 * 
 */

public class ArraySet implements Set {

	/**
	 * Rappresenta gli elementi dell'insieme.
	 */
	private String[] elements;

	/**
	 * Costruisce un insieme con gli elementi forniti.
	 * 
	 * @param elements
	 *            elementi che appartengono all'insieme.
	 */
	public ArraySet(String... elements) {
		this.elements = new String[0];
		// aggiungiamo gli elementi, ma evitando le ripetizioni
		for (String e : elements)
			if (!contains(e)) {
				String[] newElements = new String[this.elements.length + 1];
				for (int pos = 0; pos < this.elements.length; pos++)
					newElements[pos] = this.elements[pos];
				newElements[this.elements.length] = e;
				this.elements = newElements;
			}
	}

	@Override
	public String[] getElements() {
		// per evitare possibili modifiche all'insieme,
		// restituiamo una copia dell'array elements
		String[] result = new String[elements.length];
		for (int i = 0; i < elements.length; i++)
			result[i] = elements[i];
		return result;

	}

	/**
	 * Modifica <code>elements</code> assegnandovi l'array passato.
	 * 
	 * @param elements
	 *            nuovo valore per il campo <code>elements</code>.
	 */
	protected void setElements(String[] elements) {
		// assumiano che elements non abbia ripetizioni
		this.elements = elements;
	}

	@Override
	public boolean contains(String element) {
		for (String e : elements)
			if (e.equals(element))
				return true;

		return false;
	}

	@Override
	public boolean intersects(Set other) {
		for (String e : elements)
			if (other.contains(e))
				return true;

		return false;
	}

	@Override
	public int size() {
		return elements.length;
	}

	/**
	 * Verifica l'uguaglianza canonica tra insiemi. Due insiemi sono uguali se e
	 * solo se contengono gli stessi elementi.
	 * 
	 * @param that
	 *            l'oggetto da confrontare.
	 * @return <code>true</code> se
	 *         <code>that<code> e' una istanza di <code>Set</code> e contiene
	 *         gli stessi elementi di <code>this</code>; <code>false</code>
	 *         altrimenti.
	 */
	@Override
	public boolean equals(Object that) {
		if (that instanceof Set) {
			Set otherSet = (Set) that;
			if (size() != otherSet.size())
				return false;
			for (String element : elements)
				if (!otherSet.contains(element))
					return false;
			return true;
		}
		return false;
	}

}
