package bioinformatica.programmazione.esami.giugno.terzaparte;

/**
 * Classe concreta che rappresenta insiemi modificabili di stringe. E' possibile
 * aggiungere e rimuovere elementi.
 * 
 * @author mace
 * @version 1.0
 * 
 */

public class ModifiableArraySet extends ArraySet implements ModifiableSet {

	/**
	 * Crea un insieme modificabile vuoto.
	 */
	public ModifiableArraySet() {
		super();
	}

	/**
	 * Crea un insieme modificabile che contiene gli elementi indicati.
	 * 
	 * @param elements
	 *            elementi iniziali dell'insieme.
	 */
	public ModifiableArraySet(String... elements) {
		super(elements);
	}

	/**
	 * Crea un insieme modificabile che contiene gli stessi elementi
	 * dell'insieme indicato.
	 * 
	 * @param father
	 *            insieme che contiene gli elementi da includere.
	 */
	public ModifiableArraySet(Set father) {
		for (String element : father.getElements())
			add(element);
	}

	@Override
	public boolean add(String element) {
		if (contains(element))
			return false;

		String[] newElements = new String[size() + 1];
		int pos = 0;
		for (String e : this.getElements())
			newElements[pos++] = e;
		newElements[pos] = element;

		setElements(newElements);

		return true;
	}

	@Override
	public boolean remove(String element) {
		if (!contains(element))
			return false;

		String[] newElements = new String[size() - 1];
		int pos = 0;
		for (String e : this.getElements())
			if (!e.equals(element))
				newElements[pos++] = e;

		setElements(newElements);

		return true;
	}

	@Override
	public boolean addAll(Set set) {
		boolean added = false;

		for (String element : set.getElements())
			added |= add(element);

		return added;
	}

	@Override
	public boolean removeAll(Set set) {
		boolean removed = false;

		for (String element : set.getElements())
			removed |= remove(element);

		return removed;
	}

}
